# Basic Auth module for rack. Maintains backwords compatibility for the Dashboard ENC.
# Does not yet allow external auth to be used over Basic Auth, but that would be a great addition.

module Rack

  module ConsoleAuth
    class BasicAuth

      def initialize(app)
        @app = app
      end

      def call(env)
        auth = Rack::Auth::Basic::Request.new(env)
        request = Rack::Request.new(env)

        unless (auth.provided? and auth.basic? and auth.credentials)
          return @app.call(env)
        end

        ::ConsoleAuth.logger.info "Authentication method is Basic Auth. Skipping session management"

        username, password = auth.credentials
        env['rack.session']['rack.basic.user'] = username

        auth_user = ::ConsoleAuth.get_user(env)
        if auth_user
          user = User.find_by_username(auth_user.username)
        else
          user = nil
        end

        # Unless there is a local user, the request is denied.
        unless user
          ::ConsoleAuth.logger.error "Unauthorized request to #{request.path} from IP address #{request.ip}. User (#{username}) not found"
          return [401, {}, '<h1>Unauthorized</h1>']
        end

        # Check the password.
        if auth_user and auth_user.status == 'enabled' and valid_password? :user => user, :password => password
          ::ConsoleAuth.logger.info "Successful authentiation by user #{username} from IP address #{Rack::Request.new(env).ip}"
          return @app.call env
        else
          ::ConsoleAuth.logger.error "Failed authentiation by user #{username} from IP address #{Rack::Request.new(env).ip}"
          return [401, {}, '<h1>Unauthorized</h1>']
        end
      end

      def valid_password?(params)
        user = params[:user]
        pass = params[:password]

        user.encrypted_password == Digest::SHA256.hexdigest("#{user.encryption_salt}::#{pass}")
      end

    end
  end
end
