
define(['underscore', 'require', 'backbone_sandbox', 'core', 'marionette', 'stickit', 'core_lib/view_extensions', 'bootstrap'],
  function (_, require, sandbox, core, Marionette, stickit, ViewExtensions, _bootstrap) {

  var console_sandbox = Object.create(sandbox);

  // Replace the Aura defined jQuery(!) utils with the more consistent Underscore equivalents
  console_sandbox.util = {
    each: _.each,
    extend: _.extend
  };

  console_sandbox.template = core.template;
  console_sandbox.paths = core.paths;

  // We're about to replace the Aura view with a Marionette
  // equivalent, so keep a pointer to the Backbone view as well
  console_sandbox.mvc.BackboneView = console_sandbox.mvc.View;

  // Inject a "core" object in to each view, so that templates have access to important
  // information such as a users authorisation settings
  var old_serialize = Marionette.View.prototype.serializeData;

  _.extend(Marionette.View.prototype, {

    core: core,

    serializeData: function() {
      var data = old_serialize.call(this);

      if (data.core) {
        throw new Error('A `core` property already exists in this views data and it will overwrite the core defined property');
      }

      data.core = {
        user: ViewExtensions.user.inject(this),
        paths: this.core.paths
      };

      return data;
    }
  });

  // Include the Marionette views in our MVC container
  _.each([
    'CollectionView',
    'CompositeView',
    'ItemView',
    'Layout',
    'View'
  ], function(marionette_view) {
    console_sandbox.mvc[marionette_view] = function(view) {
      return Marionette[marionette_view].extend(view);
    }
  });

  // Likewise the Marionette Region and event helpers
  console_sandbox.mvc.Region = Marionette.Region;

  _.extend(console_sandbox.events, {
    Callbacks: Marionette.Callbacks,
    EventBinder: Marionette.EventBinder
  });

  // Override the default Underscore templating of Marionette with Handlebars
  _.extend(Marionette.TemplateCache.prototype, {
    compileTemplate: function(template) {
      return console_sandbox.template.parse(template);
    },

    loadTemplate: function(template) {
      return template;
    }
  });

  console_sandbox.io = {

    // load a stylesheet, with a URL relative to the base path (js/)
    // @TODO look in to creating a Require.js module path for each application
    load_stylesheet: function(stylesheet_path) {
      var url = require.toUrl(stylesheet_path);

      var link = document.createElement("link");
      link.type = "text/css";
      link.rel = "stylesheet";
      link.href = url;
      document.getElementsByTagName("head")[0].appendChild(link);
    }
  };

  return console_sandbox;
});